
#include "treap.h"
#include <iostream>
#include <set>
#include <vector>

using namespace std;

auto time(const std::function<void(void)> &func) {
    std::chrono::steady_clock::time_point begin = std::chrono::steady_clock::now();
    func();
    std::chrono::steady_clock::time_point end = std::chrono::steady_clock::now();

    return end - begin;
}


int main() {
    const auto testSize = 4000000;
    const auto testValues = 10;
    const auto testRepeat = 15;

    for (float testFl = 2; testFl < testSize; testFl *= 1.5) {
        int test = int(testFl);
        auto setDuration = 0ns;
        auto treeDuration = 0ns;

        for (int exp = 0; exp < testRepeat; exp++) {
            Treap<int> treeSet;
            treeSet.reserve(testSize);
            set<int> stdSet;


            for (int i = 0; i < test; i++) {
                auto value = rand();
                treeSet.insert(value);
                stdSet.insert(value);
            }

            vector<int> testValuesVector;
            testValuesVector.reserve(testValues);
            for (int i = 0; i < testValues; i++) {
                testValuesVector.push_back(rand());
            }

            vector<int> res1, res2;
            res1.reserve(testValuesVector.size());
            res2.reserve(testValuesVector.size());

            setDuration += time([&testValuesVector, &stdSet, &res2] {
                for (const auto &val: testValuesVector) {
                    res2.push_back(stdSet.find(val) != stdSet.end());
                }
            });

            treeDuration += time([&testValuesVector, &treeSet, &res1] {
                for (const auto &val: testValuesVector) {
                    res1.push_back(treeSet.find(val) != nullptr);
                }
            });

            for (int i = 0; i < testValuesVector.size(); i++) {
                if (res2[i] != res1[i]) {
                    cout << "Wrong work\n";
                }
            }
        }

        std::cout << test << ","
                  << std::chrono::duration_cast<std::chrono::nanoseconds>(treeDuration / testRepeat).count() << ","
                  << std::chrono::duration_cast<std::chrono::nanoseconds>(setDuration / testRepeat).count()
                  << std::endl;
    }
}