#include "treap.h"
#include <iostream>
#include <set>
#include <vector>

using namespace std;

auto time(const std::function<void(void)> &func) {
    std::chrono::steady_clock::time_point begin = std::chrono::steady_clock::now();
    func();
    std::chrono::steady_clock::time_point end = std::chrono::steady_clock::now();

    return end - begin;
}


int main() {
    const auto testSize = 10000000;
    const auto testValues = 20;
    const auto testRepeat = 20;

    for (int test = 2; test < testSize; test *= 2) {
        auto setDuration = 0ns;
        auto treeDuration = 0ns;

        for (int exp = 0; exp < testRepeat; exp++) {
            Treap<size_t> treeSet;
            treeSet.reserve(testSize);
            set<size_t> stdSet;


            for (int i = 0; i < test; i++) {
                auto value = rand();
                treeSet.insert(value);
                stdSet.insert(value);
            }

            vector<int> testValuesVector;
            testValuesVector.reserve(testValues);
            for (int i = 0; i < testValues; i++) {
                testValuesVector.push_back(rand());
            }

            setDuration += time([&testValuesVector, &stdSet] {
                for (const auto &val: testValuesVector) {
                    stdSet.insert(val);
                }
            });

            treeDuration += time([&testValuesVector, &treeSet] {
                for (const auto &val: testValuesVector) {
                    treeSet.insert(val);
                }
            });

            for (const auto &val: testValuesVector) {
                if (treeSet.find(val) == nullptr || stdSet.find(val) == stdSet.end()) {
                    cout << "FAIL";
                }
            }
        }

        std::cout << test << ","
                  << std::chrono::duration_cast<std::chrono::nanoseconds>(treeDuration / testRepeat).count() << ","
                  << std::chrono::duration_cast<std::chrono::nanoseconds>(setDuration / testRepeat).count() << std::endl;
    }
}
